from typing import Optional, Union

from mag_tools.utils.common.string_utils import StringUtils
from mag_tools.model.base_enum import BaseEnum

from mag_test.model.action_type import ActionType


class ControlType(BaseEnum):
    default_action: ActionType

    """
    控件类型枚举
    枚举值为不包含前缀的控件类型名，如：ControlType.EDIT
    """

    EDIT = ('Edit', '文本框', ActionType.SET_TEXT)  # TextBox
    DOC = ('Document', '文档', ActionType.SEND_KEYS)  # Document
    BUTTON = ('Button', '按钮', ActionType.CLICK)  # Button
    SPLIT_BUTTON = ('SplitButton', '拆分按钮', ActionType.CLICK)  # SplitButton
    CHECKBOX = ('CheckBox', '复选框', ActionType.CLICK)  # CheckBox
    RADIO = ('RadioButton', '单选按钮', ActionType.CLICK)  # RadioButton
    MENU_BAR = ('MenuBar', '菜单栏', ActionType.CLICK)  # MenuBar
    MENU = ('Menu', '菜单', ActionType.CLICK)  # Menu
    MENU_ITEM = ('MenuItem', '菜单项', ActionType.CLICK)  # MenuItem
    CONTEXT_MENU = ('ContextMenu', '上下文菜单', ActionType.CLICK)  # ContextMenu
    WINDOW = ('Window', '主窗口', ActionType.CLICK)  # Main Window
    DIALOG = ('Dialog', '对话框', ActionType.CLICK)  # Dialog
    MESSAGE = ('MessageBox', '消息框', ActionType.CLICK)  # MessageBox
    LABEL = ('Text', '标签', ActionType.CLICK)  # Label
    LIST = ('List', '列表框', ActionType.CLICK)  # ListBox
    LIST_VIEW = ('ListView', '列表视图', ActionType.CLICK)  # ListView
    LIST_ITEM = ('ListItem', '列表项', ActionType.CLICK)  # ListBox/ListView包含ListItem
    COMBO_BOX = ('ComboBox', '组合框', ActionType.CLICK)  # ComboBox
    TREE = ('Tree', '树视图', ActionType.CLICK)  # TreeView
    TREE_ITEM = ('TreeItem', '树节点', ActionType.CLICK)  # TreeItem
    TABLE = ('Table', '表格', ActionType.CLICK)    # Table
    TABLE_ROW = ('TableRow', '表格行', ActionType.CLICK) # TableRow
    DATA_ITEM = ('DataItem', '数据项', ActionType.CLICK) # DataItem
    TAB = ('Tab', '选项卡', ActionType.CLICK)  # TabControl
    TAB_ITEM = ('TabItem', 'Tab项', ActionType.CLICK)  # Tab项
    GROUP_TAB = ('GroupTab', '组TabItem', ActionType.CLICK)  # 组TabItem
    DATETIME = ('SysDateTimePick32', '日期时间', ActionType.CLICK)  # 类名为 SysDateTimePick32
    PROGRESS = ('ProgressBar', '进度条', ActionType.CLICK)  # ProgressBar
    TITLE = ('TitleBar', '标题栏', ActionType.CLICK)  # TitleBar
    SLIDER = ('Slider', '滑块', ActionType.CLICK)  # Slider
    STATUS = ('StatusBar', '状态条', ActionType.CLICK)  # StatusBar
    TOOL = ('ToolBar', '工具栏', ActionType.CLICK)  # ToolBar
    GROUP = ('Group', '组', ActionType.CLICK)  # 组Group
    PANEL = ('Panel', 'Panel', ActionType.CLICK)  # Panel 分组和布局
    PANE = ('Pane', 'Pane', ActionType.CLICK)  # Panel 分组框或面板
    HEADER = ('Header', '表格标题', ActionType.CLICK)  # Header
    FILE = ('File', '文件', ActionType.CLEAR_DIR) # 文件操作，虚拟控件
    ALL = ('*', '全部', None) # 全部控件

    def __init__(self, code: Optional[Union[str, int]] = None, desc: Optional[str] = None, default_action: Optional[ActionType] = None):
        super(BaseEnum, self).__init__(code, desc)
        self.default_action = default_action

    @classmethod
    def get_by_element(cls, element):
        if element is None:
            return None

        type_name = StringUtils.get_after_keyword(element.tag_name, ".")
        return ControlType.of_code(type_name)

    def is_composite(self):
        """
        判定是否为复合控件
        """
        return self in {ControlType.MENU, ControlType.COMBO_BOX,
                        ControlType.LIST, ControlType.LIST_VIEW, ControlType.TREE, ControlType.PANE,
                        ControlType.TOOL, ControlType.DATETIME, ControlType.WINDOW}

    def is_virtual(self):
        """
        判定是否为虚拟控件
        """
        return self in {ControlType.FILE}
